/*
 * $URL$
 * $Id$
 *
 * Version 1.7
 *
 * A Java class to determine image width, height and color depth for
 * a number of image file formats.
 *
 * Written by Marco Schmidt
 *
 * Contributed to the Public Domain.
 */
package com.day.cq.dam.handler.standard.helper.imageinfo;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.DataInput;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

/**
 * Get file format, image resolution, number of bits per pixel and optionally
 * number of images, comments and physical resolution from
 * JPEG, GIF, BMP, PCX, PNG, IFF, RAS, PBM, PGM, PPM and PSD files
 * (or input streams).
 * <p>
 * Use the class like this:
 * <pre>
 * ImageInfo ii = new ImageInfo();
 * ii.setInput(in); // in can be InputStream or RandomAccessFile
 * ii.setDetermineImageNumber(true); // default is false
 * ii.setCollectComments(true); // default is false
 * if (!ii.check()) {
 *   System.err.println("Not a supported image file format.");
 *   return;
 * }
 * System.out.println(ii.getFormatName() + ", " + ii.getMimeType() +
 *   ", " + ii.getWidth() + " x " + ii.getHeight() + " pixels, " +
 *   ii.getBitsPerPixel() + " bits per pixel, " + ii.getNumberOfImages() +
 *   " image(s), " + ii.getNumberOfComments() + " comment(s).");
 *  // there are other properties, check out the API documentation
 * </pre>
 * You can also use this class as a command line program.
 * Call it with a number of image file names and URLs as parameters:
 * <pre>
 *   java ImageInfo *.jpg *.png *.gif http://somesite.tld/image.jpg
 * </pre>
 * or call it without parameters and pipe data to it:
 * <pre>
 *   java ImageInfo &lt; image.jpg
 * </pre>
 * <p>
 * Known limitations:
 * <ul>
 * <li>When the determination of the number of images is turned off, GIF bits
 *  per pixel are only read from the global header.
 *  For some GIFs, local palettes change this to a typically larger
 *  value. To be certain to get the correct color depth, call
 *  setDetermineImageNumber(true) before calling check().
 *  The complete scan over the GIF file will take additional time.</li>
 * <li>Transparency information is not included in the bits per pixel count.
 *  Actually, it was my decision not to include those bits, so it's a feature! ;-)</li>
 * </ul>
 * <p>
 * Requirements:
 * <ul>
 * <li>Java 1.1 or higher</li>
 * </ul>
 * <p>
 * The latest version can be found at <a href="http://schmidt.devlib.org/image-info/">http://schmidt.devlib.org/image-info/</a>.
 * <p>
 * Written by Marco Schmidt.
 * <p>
 * This class is contributed to the Public Domain.
 * Use it at your own risk.
 * <p>
 * Last modification 2005-07-26.
 * <p>
 * <a name="history">History</a>:
 * <ul>
 * <li><strong>2001-08-24</strong> Initial version.</li>
 * <li><strong>2001-10-13</strong> Added support for the file formats BMP and PCX.</li>
 * <li><strong>2001-10-16</strong> Fixed bug in read(int[], int, int) that returned
 * <li><strong>2002-01-22</strong> Added support for file formats Amiga IFF and Sun Raster (RAS).</li>
 * <li><strong>2002-01-24</strong> Added support for file formats Portable Bitmap / Graymap / Pixmap (PBM, PGM, PPM) and Adobe Photoshop (PSD).
 *   Added new method getMimeType() to return the MIME type associated with a particular file format.</li>
 * <li><strong>2002-03-15</strong> Added support to recognize number of images in file. Only works with GIF.
 *   Use {@link #setDetermineImageNumber} with <code>true</code> as argument to identify animated GIFs
 *   ({@link #getNumberOfImages()} will return a value larger than <code>1</code>).</li>
 * <li><strong>2002-04-10</strong> Fixed a bug in the feature 'determine number of images in animated GIF' introduced with version 1.1.
 *   Thanks to Marcelo P. Lima for sending in the bug report.
 *   Released as 1.1.1.</li>
 * <li><strong>2002-04-18</strong> Added {@link #setCollectComments(boolean)}.
 *  That new method lets the user specify whether textual comments are to be
 *  stored in an internal list when encountered in an input image file / stream.
 *  Added two methods to return the physical width and height of the image in dpi:
 *   {@link #getPhysicalWidthDpi()} and {@link #getPhysicalHeightDpi()}.
 *  If the physical resolution could not be retrieved, these methods return <code>-1</code>.
 *  </li>
 * <li><strong>2002-04-23</strong> Added support for the new properties physical resolution and
 *   comments for some formats. Released as 1.2.</li>
 * <li><strong>2002-06-17</strong> Added support for SWF, sent in by Michael Aird.
 *  Changed checkJpeg() so that other APP markers than APP0 will not lead to a failure anymore.
 *  Released as 1.3.</li>
 * <li><strong>2003-07-28</strong> Bug fix - skip method now takes return values into consideration.
 *  Less bytes than necessary may have been skipped, leading to flaws in the retrieved information in some cases.
 *  Thanks to Bernard Bernstein for pointing that out.
 *  Released as 1.4.</li>
 * <li><strong>2004-02-29</strong> Added support for recognizing progressive JPEG and
 *  interlaced PNG and GIF. A new method {@link #isProgressive()} returns whether ImageInfo
 *  has found that the storage type is progressive (or interlaced).
 *  Thanks to Joe Germuska for suggesting the feature.
 *  Bug fix: BMP physical resolution is now correctly determined.
 *  Released as 1.5.</li>
 * <li><strong>2004-11-30</strong> Bug fix: recognizing progressive GIFs
 * (interlaced in GIF terminology) did not work (thanks to Franz Jeitler for
 *   pointing this out). Now it should work, but only if the number of images is determined.
 *  This is because information on interlacing is stored in a local image header.
 *  In theory, different images could be stored interlaced and non-interlaced in one
 *  file. However, I think  that's unlikely. Right now, the last image in the GIF file
 *  that is examined by ImageInfo is used for the "progressive" status.</li>
 * <li><strong>2005-01-02</strong> Some code clean up (unused methods and variables
 *  commented out, missing javadoc comments, etc.). Thanks to George Sexton for a long list.
 *  Removed usage of Boolean.toString because
 *  it's a Java 1.4+ feature (thanks to Gregor Dupont).
 *  Changed delimiter character in compact output from semicolon to tabulator
 * (for better integration with cut(1) and other Unix tools).
 *  Added some points to the <a href="http://schmidt.devlib.org/image-info/index.html#knownissues">'Known
 *  issues' section of the website</a>.
 *  Released as 1.6.</li>
 * <li><strong>2005-07-26</strong> Removed code to identify Flash (SWF) files.
 *  Has repeatedly led to problems and support requests, and I don't know the
 *  format and don't have the time and interest to fix it myself.
 *  I repeatedly included fixes by others which didn't work for some people.
 *  I give up on SWF. Please do not contact me about it anymore.
 *  Set package of ImageInfo class to org.devlib.schmidt.imageinfo (a package
 *  was repeatedly requested by some users).
 *  Released as 1.7.</li>
 * </ul>
 */
public class ImageInfo {
    /**
     * The Logger
     */
    private static final Logger log = LoggerFactory.getLogger(ImageInfo.class);

	/**
	 * Return value of {@link #getFormat()} for JPEG streams.
	 * ImageInfo can extract physical resolution and comments
	 * from JPEGs (only from APP0 headers).
	 * Only one image can be stored in a file.
	 * It is determined whether the JPEG stream is progressive
	 * (see {@link #isProgressive()}).
	 */
	public static final int FORMAT_JPEG = 0;

	/**
	 * Return value of {@link #getFormat()} for GIF streams.
	 * ImageInfo can extract comments from GIFs and count the number
	 * of images (GIFs with more than one image are animations).
	 * It is determined whether the GIF stream is interlaced (see {@link #isProgressive()}).
	 */
	public static final int FORMAT_GIF = 1;

	/**
	 * Return value of {@link #getFormat()} for PNG streams.
	 * PNG only supports one image per file.
	 * Both physical resolution and comments can be stored with PNG,
	 * but ImageInfo is currently not able to extract those.
	 * It is determined whether the PNG stream is interlaced (see {@link #isProgressive()}).
	 */
	public static final int FORMAT_PNG = 2;

	/**
	 * Return value of {@link #getFormat()} for BMP streams.
	 * BMP only supports one image per file.
	 * BMP does not allow for comments.
	 * The physical resolution can be stored.
	 */
	public static final int FORMAT_BMP = 3;

	/**
	 * Return value of {@link #getFormat()} for PCX streams.
	 * PCX does not allow for comments or more than one image per file.
	 * However, the physical resolution can be stored.
	 */
	public static final int FORMAT_PCX = 4;

	/**
	 * Return value of {@link #getFormat()} for IFF streams.
	 */
	public static final int FORMAT_IFF = 5;

	/**
	 * Return value of {@link #getFormat()} for RAS streams.
	 * Sun Raster allows for one image per file only and is not able to
	 * store physical resolution or comments.
	 */
	public static final int FORMAT_RAS = 6;

	/** Return value of {@link #getFormat()} for PBM streams. */
	public static final int FORMAT_PBM = 7;

	/** Return value of {@link #getFormat()} for PGM streams. */
	public static final int FORMAT_PGM = 8;

	/** Return value of {@link #getFormat()} for PPM streams. */
	public static final int FORMAT_PPM = 9;

	/** Return value of {@link #getFormat()} for PSD streams. */
	public static final int FORMAT_PSD = 10;

/*	public static final int COLOR_TYPE_UNKNOWN = -1;
	public static final int COLOR_TYPE_TRUECOLOR_RGB = 0;
	public static final int COLOR_TYPE_PALETTED = 1;
	public static final int COLOR_TYPE_GRAYSCALE= 2;
	public static final int COLOR_TYPE_BLACK_AND_WHITE = 3;*/

	/**
	 * The names of all supported file formats.
	 * The FORMAT_xyz int constants can be used as index values for
	 * this array.
	 */
	private static final String[] FORMAT_NAMES =
		{"JPEG", "GIF", "PNG", "BMP", "PCX",
		 "IFF", "RAS", "PBM", "PGM", "PPM",
		 "PSD"};

	/**
	 * The names of the MIME types for all supported file formats.
	 * The FORMAT_xyz int constants can be used as index values for
	 * this array.
	 */
	private static final String[] MIME_TYPE_STRINGS =
		{"image/jpeg", "image/gif", "image/png", "image/bmp", "image/pcx",
		 "image/iff", "image/ras", "image/x-portable-bitmap", "image/x-portable-graymap", "image/x-portable-pixmap",
		 "image/psd"};

	private int width;
	private int height;
	private int bitsPerPixel;
	//private int colorType = COLOR_TYPE_UNKNOWN;
	private boolean progressive;
	private int format;
	private InputStream in;
	private DataInput din;
	private boolean collectComments = true;
	private Vector comments;
	private boolean determineNumberOfImages;
	private int numberOfImages;
	private int physicalHeightDpi;
	private int physicalWidthDpi;
	private int bitBuf;
	private int bitPos;

	private void addComment(String s) {
	}

	/**
	 * Call this method after you have provided an input stream or file
	 * using {@link #setInput(InputStream)} or {@link #setInput(DataInput)}.
	 * If true is returned, the file format was known and information
	 * on the file's content can be retrieved using the various getXyz methods.
	 * @return if information could be retrieved from input
	 */
	public boolean check() {
		return false;
	}

	private boolean checkBmp() throws IOException {
		return false;
	}

	private boolean checkGif() throws IOException {
		return false;
	}

	private boolean checkIff() throws IOException {
		return false;
	}

	private boolean checkJpeg() throws IOException {
		return false;
	}

	private boolean checkPcx() throws IOException {
		return false;
	}

	private boolean checkPng() throws IOException {
		return false;
	}

	private boolean checkPnm(int id) throws IOException {
		return false;
	}

	private boolean checkPsd() throws IOException {
		return false;
	}

	private boolean checkRas() throws IOException {
		return false;
	}

	/**
	 * Run over String list, return false iff at least one of the arguments
	 * equals <code>-c</code>.
	 * @param args string list to check
	 */
	private static boolean determineVerbosity(String[] args) {
		return false;
	}

	private static boolean equals(byte[] a1, int offs1, byte[] a2, int offs2, int num) {
		return false;
	}

	/**
	 * If {@link #check()} was successful, returns the image's number of bits per pixel.
	 * Does not include transparency information like the alpha channel.
	 * @return number of bits per image pixel
	 */
	public int getBitsPerPixel() {
		return 0;
	}

	/**
	 * Returns the index'th comment retrieved from the file.
	 * @param index int index of comment to return
	 * @throws IllegalArgumentException if index is smaller than 0 or larger than or equal
	 * to the number of comments retrieved
	 * @see #getNumberOfComments
	 */
	public String getComment(int index) {
		return null;
	}

	/**
	 * If {@link #check()} was successful, returns the image format as one
	 * of the FORMAT_xyz constants from this class.
	 * Use {@link #getFormatName()} to get a textual description of the file format.
	 * @return file format as a FORMAT_xyz constant
	 */
	public int getFormat() {
		return 0;
	}

	/**
	 * If {@link #check()} was successful, returns the image format's name.
	 * Use {@link #getFormat()} to get a unique number.
	 * @return file format name
	 */
	public String getFormatName() {
		return null;
	}

	/**
	 * If {@link #check()} was successful, returns one the image's vertical
	 * resolution in pixels.
	 * @return image height in pixels
	 */
	public int getHeight() {
		return 0;
	}

	private static int getIntBigEndian(byte[] a, int offs) {
		return 0;
	}

	private static int getIntLittleEndian(byte[] a, int offs) {
		return 0;
	}

	/**
	 * If {@link #check()} was successful, returns a String with the
	 * MIME type of the format.
	 * @return MIME type, e.g. <code>image/jpeg</code>
	 */
	public String getMimeType() {
		return null;
	}

	/**
	 * If {@link #check()} was successful and {@link #setCollectComments(boolean)} was called with
	 * <code>true</code> as argument, returns the number of comments retrieved
	 * from the input image stream / file.
	 * Any number &gt;= 0 and smaller than this number of comments is then a
	 * valid argument for the {@link #getComment(int)} method.
	 * @return number of comments retrieved from input image
	 */
	public int getNumberOfComments()
	{
		return 0;
	}

	/**
	 * Returns the number of images in the examined file.
	 * Assumes that <code>setDetermineImageNumber(true);</code> was called before
	 * a successful call to {@link #check()}.
	 * This value can currently be only different from <code>1</code> for GIF images.
	 * @return number of images in file
	 */
	public int getNumberOfImages()
	{
		return 0;
	}

	/**
	 * Returns the physical height of this image in dots per inch (dpi).
	 * Assumes that {@link #check()} was successful.
	 * Returns <code>-1</code> on failure.
	 * @return physical height (in dpi)
	 * @see #getPhysicalWidthDpi()
	 * @see #getPhysicalHeightInch()
	 */
	public int getPhysicalHeightDpi() {
		return 0;
	}

	/**
	 * If {@link #check()} was successful, returns the physical width of this image in dpi (dots per inch)
	 * or -1 if no value could be found.
	 * @return physical height (in dpi)
	 * @see #getPhysicalHeightDpi()
	 * @see #getPhysicalWidthDpi()
	 * @see #getPhysicalWidthInch()
	 */
	public float getPhysicalHeightInch() {
		return 0.0f;
	}

	/**
	 * If {@link #check()} was successful, returns the physical width of this image in dpi (dots per inch)
	 * or -1 if no value could be found.
	 * @return physical width (in dpi)
	 * @see #getPhysicalHeightDpi()
	 * @see #getPhysicalWidthInch()
	 * @see #getPhysicalHeightInch()
	 */
	public int getPhysicalWidthDpi() {
		return 0;
	}

	/**
	 * Returns the physical width of an image in inches, or
	 * <code>-1.0f</code> if width information is not available.
	 * Assumes that {@link #check} has been called successfully.
	 * @return physical width in inches or <code>-1.0f</code> on failure
	 * @see #getPhysicalWidthDpi
	 * @see #getPhysicalHeightInch
	 */
	public float getPhysicalWidthInch() {
		return 0.0f;
	}

	private static int getShortBigEndian(byte[] a, int offs) {
		return 0;
	}

	private static int getShortLittleEndian(byte[] a, int offs) {
		return 0;
	}

	/**
	 * If {@link #check()} was successful, returns one the image's horizontal
	 * resolution in pixels.
	 * @return image width in pixels
	 */
	public int getWidth() {
		return 0;
	}

	/**
	 * Returns whether the image is stored in a progressive (also called: interlaced) way.
	 * @return true for progressive/interlaced, false otherwise
	 */
	public boolean isProgressive()
	{
		return false;
	}

	/**
	 * To use this class as a command line application, give it either
	 * some file names as parameters (information on them will be
	 * printed to standard output, one line per file) or call
	 * it with no parameters. It will then check data given to it
	 * via standard input.
	 * @param args the program arguments which must be file names
	 */
	public static void main(String[] args) {
	}

	private static void print(String sourceName, ImageInfo ii, boolean verbose) {
	}

	private static void printCompact(String sourceName, ImageInfo imageInfo) {
	}

	private static void printLine(int indentLevels, String text, float value, float minValidValue) {
	}

	private static void printLine(int indentLevels, String text, int value, int minValidValue) {
	}

	private static void printLine(int indentLevels, String text, String value) {
	}

	private static void printVerbose(String sourceName, ImageInfo ii) {
	}

	private int read() throws IOException {
		return 0;
	}

	private int read(byte[] a) throws IOException {
		return 0;
	}

	private int read(byte[] a, int offset, int num) throws IOException {
		return 0;
	}

	private String readLine() throws IOException {
		return null;
	}

	private String readLine(StringBuffer sb) throws IOException {
		return null;
	}

	private long readUBits( int numBits ) throws IOException
	{
		return 0L;
	}

    /**
     * Read a signed integer value from input.
     * @param numBits number of bits to read
     */
    private int readSBits(int numBits) throws IOException
    {
		return 0;
	}

/*	private void synchBits()
	{
		bitBuf = 0;
		bitPos = 0;
	}*/

/*	private String readLine(int firstChar) throws IOException {
		StringBuffer result = new StringBuffer();
		result.append((char)firstChar);
		return readLine(result);
	}*/

	private static void run(String sourceName, InputStream in, ImageInfo imageInfo, boolean verbose) {
	}

	/**
	 * Specify whether textual comments are supposed to be extracted from input.
	 * Default is <code>false</code>.
	 * If enabled, comments will be added to an internal list.
	 * @param newValue if <code>true</code>, this class will read comments
	 * @see #getNumberOfComments
	 * @see #getComment
	 */
	public void setCollectComments(boolean newValue)
	{
	}

	/**
	 * Specify whether the number of images in a file is to be
	 * determined - default is <code>false</code>.
	 * This is a special option because some file formats require running over
	 * the entire file to find out the number of images, a rather time-consuming
	 * task.
	 * Not all file formats support more than one image.
	 * If this method is called with <code>true</code> as argument,
	 * the actual number of images can be queried via
	 * {@link #getNumberOfImages()} after a successful call to
	 * {@link #check()}.
	 * @param newValue will the number of images be determined?
	 * @see #getNumberOfImages
	 */
	public void setDetermineImageNumber(boolean newValue)
	{
	}

	/**
	 * Set the input stream to the argument stream (or file).
	 * Note that {@link java.io.RandomAccessFile} implements
	 * {@link java.io.DataInput}.
	 * @param dataInput the input stream to read from
	 */
	public void setInput(DataInput dataInput) {
	}

	/**
	 * Set the input stream to the argument stream (or file).
	 * @param inputStream the input stream to read from
	 */
	public void setInput(InputStream inputStream) {
	}

	private void setPhysicalHeightDpi(int newValue) {
	}

	private void setPhysicalWidthDpi(int newValue) {
	}

	private void skip(int num) throws IOException {
	}
}

