/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.blob.datastore;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Iterators.filter;
import static com.google.common.collect.Iterators.transform;
import static org.apache.commons.io.IOUtils.closeQuietly;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.SequenceInputStream;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.jcr.RepositoryException;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.cache.LoadingCache;
import com.google.common.cache.Weigher;
import com.google.common.collect.Iterators;
import com.google.common.collect.Lists;
import com.google.common.io.ByteStreams;
import com.google.common.io.Closeables;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.jackrabbit.core.data.DataIdentifier;
import org.apache.jackrabbit.core.data.DataRecord;
import org.apache.jackrabbit.core.data.DataStore;
import org.apache.jackrabbit.core.data.DataStoreException;
import org.apache.jackrabbit.core.data.MultiDataStoreAware;
import org.apache.jackrabbit.oak.cache.CacheLIRS;
import org.apache.jackrabbit.oak.cache.CacheStats;
import org.apache.jackrabbit.oak.commons.StringUtils;
import org.apache.jackrabbit.oak.plugins.blob.BlobTrackingStore;
import org.apache.jackrabbit.oak.plugins.blob.SharedDataStore;
import org.apache.jackrabbit.oak.spi.blob.BlobOptions;
import org.apache.jackrabbit.oak.spi.blob.BlobStore;
import org.apache.jackrabbit.oak.spi.blob.stats.StatsCollectingStreams;
import org.apache.jackrabbit.oak.spi.blob.stats.BlobStatsCollector;
import org.apache.jackrabbit.oak.spi.blob.GarbageCollectableBlobStore;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * BlobStore wrapper for DataStore. Wraps Jackrabbit 2 DataStore and expose them as BlobStores
 * It also handles inlining binaries if there size is smaller than
 * {@link org.apache.jackrabbit.core.data.DataStore#getMinRecordLength()}
 */
public class DataStoreBlobStore
    implements DataStore, BlobStore, GarbageCollectableBlobStore, BlobTrackingStore, TypedDataStore {
    private final Logger log = LoggerFactory.getLogger(getClass());

    protected final DataStore delegate;

    protected BlobStatsCollector stats = BlobStatsCollector.NOOP;

    private BlobTracker tracker;

    /**
     * If set to true then the blob length information would be encoded as part of blobId
     * and thus no extra call would be made to DataStore to determine the length
     *
     * <b>Implementation Note</b>If enabled the length would be encoded in blobid by appending it at the end.
     * This would be done for the methods which are part of BlobStore and GarbageCollectableBlobStore interface
     *
     * DataIdentifiers which are part of DataStore would not be affected by this as DataStore interface
     * is not used in Oak and all access is via BlobStore interface
     */
    private final boolean encodeLengthInId;

    protected final LoadingCache<String, byte[]> cache;

    public static final int DEFAULT_CACHE_SIZE = 16;

    /**
     * Max size of binary whose content would be cached. We keep it greater than
     * Lucene blob size OakDirectory#BLOB_SIZE such that Lucene index blobs are cached
     */
    private int maxCachedBinarySize = 1024 * 1024;

    private final Weigher<String, byte[]> weigher = new Weigher<String, byte[]>() {
        @Override
        public int weigh(@Nonnull String key, @Nonnull byte[] value) {
			return 0;
		}
    };

    private final CacheStats cacheStats;

    public static final String MEM_CACHE_NAME = "BlobStore-MemCache";

    public DataStoreBlobStore(DataStore delegate) {
        this(delegate, true, DEFAULT_CACHE_SIZE);
    }

    public DataStoreBlobStore(DataStore delegate, boolean encodeLengthInId) {
        this(delegate, encodeLengthInId, DEFAULT_CACHE_SIZE);
    }

    public DataStoreBlobStore(DataStore delegate, boolean encodeLengthInId, int cacheSizeInMB) {
        this.delegate = delegate;
        this.encodeLengthInId = encodeLengthInId;

        long cacheSize = (long) cacheSizeInMB * FileUtils.ONE_MB;
        this.cache = CacheLIRS.<String, byte[]>newBuilder()
                .module(MEM_CACHE_NAME)
                .recordStats()
                .maximumWeight(cacheSize)
                .weigher(weigher)
                .build();
        this.cacheStats = new CacheStats(cache, MEM_CACHE_NAME, weigher, cacheSize);
    }

    //~----------------------------------< DataStore >

    @Override
    public DataRecord getRecordIfStored(DataIdentifier identifier) throws DataStoreException {
		return null;
	}

    @Override
    public DataRecord getRecord(DataIdentifier identifier) throws DataStoreException {
		return null;
	}

    @Override
    public DataRecord getRecordFromReference(String reference) throws DataStoreException {
		return null;
	}

    @Override
    public DataRecord addRecord(InputStream stream) throws DataStoreException {
		return null;
	}

    @Override
    public void updateModifiedDateOnAccess(long before) {
	}

    @Override
    public int deleteAllOlderThan(long min) throws DataStoreException {
		return 0;
	}

    @Override
    public Iterator<DataIdentifier> getAllIdentifiers() throws DataStoreException {
		return null;
	}

    @Override
    public void init(String homeDir) throws RepositoryException {
	}

    @Override
    public int getMinRecordLength() {
		return 0;
	}

    @Override
    public void close() throws DataStoreException {
	}

    //~-------------------------------------------< BlobStore >

    @Override
    public String writeBlob(InputStream stream) throws IOException {
		return null;
	}

    @Override
    public String writeBlob(InputStream stream, BlobOptions options) throws IOException {
		return null;
	}

    @Override
    public int readBlob(String encodedBlobId, long pos, byte[] buff, int off, int length) throws IOException {
		return 0;
	}

    @Override
    public long getBlobLength(String encodedBlobId) throws IOException {
		return 0L;
	}

    @Override
    public String getBlobId(@Nonnull String reference) {
		return null;
	}

    @Override
    public String getReference(@Nonnull String encodedBlobId) {
		return null;
	}

    @Override
    public InputStream getInputStream(final String encodedBlobId) throws IOException {
		return null;
	}

    //~-------------------------------------------< GarbageCollectableBlobStore >

    @Override
    public void setBlockSize(int x) {
	}

    @Override
    public String writeBlob(String tempFileName) throws IOException {
		return null;
	}

    @Override
    public int sweep() throws IOException {
		return 0;
	}

    @Override
    public void startMark() throws IOException {
	}

    @Override
    public void clearInUse() {
	}

    @Override
    public void clearCache() {
	}

    @Override
    public long getBlockSizeMin() {
		return 0L;
	}

    @Override
    public Iterator<String> getAllChunkIds(final long maxLastModifiedTime) throws Exception {
		return null;
	}

    @Override
    public boolean deleteChunks(List<String> chunkIds, long maxLastModifiedTime) throws Exception {
		return false;
	}    
    
    @Override
    public long countDeleteChunks(List<String> chunkIds, long maxLastModifiedTime) throws Exception {
		return 0L;
	}

    @Override
    public Iterator<String> resolveChunks(String blobId) throws IOException {
		return null;
	}

    @Override
    public void addMetadataRecord(InputStream stream, String name) throws DataStoreException {
	}

    @Override
    public void addMetadataRecord(File f, String name) throws DataStoreException {
	}

    @Override public DataRecord getMetadataRecord(String name) {
		return null;
	}

    @Override
    public List<DataRecord> getAllMetadataRecords(String prefix) {
		return null;
	}

    @Override
    public boolean deleteMetadataRecord(String name) {
		return false;
	}

    @Override
    public void deleteAllMetadataRecords(String prefix) {
	}

    @Override
    public Iterator<DataRecord> getAllRecords() throws DataStoreException {
		return null;
	}

    @Override
    public DataRecord getRecordForId(DataIdentifier identifier) throws DataStoreException {
		return null;
	}

    @Override
    public Type getType() {
		return null;
	}


    @Override
    public DataRecord addRecord(InputStream input, BlobOptions options) throws DataStoreException {
		return null;
	}

    //~---------------------------------------------< Object >

    @Override
    public String toString() {
		return null;
	}

    //~---------------------------------------------< Properties >

    public DataStore getDataStore() {
		return null;
	}

    public CacheStats getCacheStats() {
		return null;
	}

    public void setMaxCachedBinarySize(int maxCachedBinarySize) {
	}

    public void setBlobStatsCollector(BlobStatsCollector stats) {
	}


    @Override
    public void addTracker(BlobTracker tracker) {
	}

    @Override
    @Nullable
    public BlobTracker getTracker() {
		return null;
	}


    //~---------------------------------------------< Internal >

    protected InputStream getStream(String blobId) throws IOException {
		return null;
	}

    protected DataRecord getDataRecord(String blobId) throws DataStoreException {
		return null;
	}

    private static boolean isInMemoryRecord(DataIdentifier identifier) {
		return false;
	}

    /**
     * Create a BLOB value from in input stream. Small objects will create an in-memory object,
     * while large objects are stored in the data store
     *
     * @param in the input stream
     * @param options
     * @return the value
     */
    private DataRecord writeStream(InputStream in, BlobOptions options) throws IOException, DataStoreException {
		return null;
	}

    private String getBlobId(DataRecord dr) {
		return null;
	}

    protected String extractBlobId(String encodedBlobId) {
		return null;
	}

    public static class BlobId {
        static final String SEP = "#";

        public String getBlobId() {
			return null;
		}

        final String blobId;
        final long length;

        BlobId(String blobId, long length) {
            this.blobId = blobId;
            this.length = length;
        }

        BlobId(DataRecord dr) {
            this.blobId = dr.getIdentifier().toString();
            long len;
            try {
                len = dr.getLength();
            } catch (DataStoreException e) {
                //Cannot determine length
                len = -1;
            }
            this.length = len;
        }

        BlobId(String encodedBlobId) {
            int indexOfSep = encodedBlobId.lastIndexOf(SEP);
            if (indexOfSep != -1) {
                this.blobId = encodedBlobId.substring(0, indexOfSep);
                this.length = Long.valueOf(encodedBlobId.substring(indexOfSep+SEP.length()));
            } else {
                this.blobId = encodedBlobId;
                this.length = -1;
            }
        }

        String encodedValue() {
			return null;
		}

        boolean hasLengthInfo() {
			return false;
		}

        static boolean isEncoded(String encodedBlobId) {
			return false;
		}

        public static BlobId of(String encodedValue) {
			return null;
		}

        static BlobId of(DataRecord dr) {
			return null;
		}
    }
}
