/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.nodetype.write;

import static org.apache.jackrabbit.JcrConstants.JCR_AUTOCREATED;
import static org.apache.jackrabbit.JcrConstants.JCR_MANDATORY;
import static org.apache.jackrabbit.JcrConstants.JCR_NAME;
import static org.apache.jackrabbit.JcrConstants.JCR_ONPARENTVERSION;
import static org.apache.jackrabbit.JcrConstants.JCR_PROTECTED;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.ItemDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.version.OnParentVersionAction;

import org.apache.jackrabbit.oak.api.Tree;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.namepath.NameMapper;
import org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants;

/**
 * Base class for the node and property definition template implementations
 * in this package. Takes care of the shared item definition attributes and
 * manages mappings between JCR and Oak names.
 */
abstract class ItemDefinitionTemplate extends NamedTemplate
        implements ItemDefinition {

    private boolean residual = false;

    private boolean isAutoCreated = false;

    private int onParentVersion = OnParentVersionAction.COPY;

    private boolean isProtected = false;

    private boolean isMandatory = false;

    protected ItemDefinitionTemplate(NameMapper mapper) {
        super(mapper);
    }

    protected ItemDefinitionTemplate(
            NameMapper mapper, ItemDefinition definition)
            throws ConstraintViolationException {
        super(mapper, definition.getName());
        setProtected(definition.isProtected());
        setMandatory(definition.isMandatory());
        setAutoCreated(definition.isAutoCreated());
        setOnParentVersion(definition.getOnParentVersion());
    }

    /**
     * Writes the contents of this item definition to the given tree node.
     * Used when registering new node types.
     *
     * @param tree an {@code nt:propertyDefinition} or
     *             {@code nt:childNodeDefinition} node
     * @throws RepositoryException if this definition could not be written
     */
    void writeTo(Tree tree) throws RepositoryException {
	}

    //------------------------------------------------------------< public >--

    /**
     * Returns the name of this template, or {@code null} if the name
     * has not yet been set. The special name "*" is used for residual
     * item definitions.
     *
     * @return JCR name, "*", or {@code null}
     */
    @Override @CheckForNull
    public String getName() {
		return null;
	}

    /**
     * Sets the name of this template. Use the special name "*" for a residual
     * item definition.
     *
     * @param jcrName JCR name, or "*"
     * @throws ConstraintViolationException if the name is invalid
     */
    @Override
    public void setName(@Nonnull String jcrName)
            throws ConstraintViolationException {
			}

    /**
     * Returns {@code null} since an item definition template is not
     * attached to a live, already registered node type.
     *
     * @return {@code null}
     */
    @Override
    public NodeType getDeclaringNodeType() {
		return null;
	}

    @Override
    public boolean isAutoCreated() {
		return false;
	}

    public void setAutoCreated(boolean isAutoCreated) {
	}

    @Override
    public boolean isMandatory() {
		return false;
	}

    public void setMandatory(boolean isMandatory) {
	}

    @Override
    public int getOnParentVersion() {
		return 0;
	}

    public void setOnParentVersion(int onParentVersion) {
	}

    @Override
    public boolean isProtected() {
		return false;
	}

    public void setProtected(boolean isProtected) {
	}

}
