/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.plugins.observation.filter;

import static java.util.Collections.disjoint;
import static org.apache.jackrabbit.oak.commons.PathUtils.concat;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.plugins.observation.ChangeSet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ChangeSetFilterImpl implements ChangeSetFilter {

    private static final Logger LOG = LoggerFactory.getLogger(ChangeSetFilterImpl.class);
    
    private static final int MAX_EXCLUDED_PATHS = 11;
    private static final int MAX_EXCLUDE_PATH_CUTOFF_LEVEL = 6;
    
    private final Set<String> rootIncludePaths;
    private final Set<String> firstLevelIncludeNames;
    private final Set<Pattern> includePathPatterns;
    private final Set<Pattern> excludePathPatterns;
    private final Set<Pattern> unpreciseExcludePathPatterns;
    private final Set<String> parentNodeNames;
    private final Set<String> parentNodeTypes;
    private final Set<String> propertyNames;
    
    @Override
    public String toString() {
		return null;
	}

    public ChangeSetFilterImpl(@Nonnull Set<String> includedParentPaths, boolean isDeep,
            @Nullable Set<String> additionalIncludedParentPaths, Set<String> excludedParentPaths,
            Set<String> parentNodeNames, Set<String> parentNodeTypes, Set<String> propertyNames) {
        this(includedParentPaths, isDeep, additionalIncludedParentPaths, excludedParentPaths, parentNodeNames, parentNodeTypes, propertyNames,
                MAX_EXCLUDED_PATHS);
    }

    public ChangeSetFilterImpl(@Nonnull Set<String> includedParentPaths, boolean isDeep,
            @Nullable Set<String> additionalIncludedParentPaths, Set<String> excludedParentPaths,
            Set<String> parentNodeNames, Set<String> parentNodeTypes, Set<String> propertyNames,
            int maxExcludedPaths) {
        this.rootIncludePaths = new HashSet<String>();
        this.includePathPatterns = new HashSet<Pattern>();
        Set<String> firstLevelIncludePaths = new HashSet<String>();
        for (String aRawIncludePath : includedParentPaths) {
            final String aGlobbingIncludePath;
            if (aRawIncludePath.contains("*")) {
                // then isDeep is not applicable, it is already a glob path
                aGlobbingIncludePath = aRawIncludePath;
            } else {
                aGlobbingIncludePath = !isDeep ? aRawIncludePath : concat(aRawIncludePath, "**");
            }
            this.rootIncludePaths.add(aRawIncludePath);
            this.includePathPatterns.add(asPattern(aGlobbingIncludePath));
            if (firstLevelIncludePaths != null) {
                final String firstLevelName = firstLevelName(aRawIncludePath);
                if (firstLevelName != null && !firstLevelName.contains("*")) {
                    firstLevelIncludePaths.add(firstLevelName);
                } else {
                    firstLevelIncludePaths = null;
                }
            }
        }
        if (additionalIncludedParentPaths != null) {
            for (String path : additionalIncludedParentPaths) {
                this.rootIncludePaths.add(path);
                this.includePathPatterns.add(asPattern(path));
                if (firstLevelIncludePaths != null) {
                    final String firstLevelName = firstLevelName(path);
                    if (firstLevelName != null && !firstLevelName.contains("*")) {
                        firstLevelIncludePaths.add(firstLevelName);
                    } else {
                        firstLevelIncludePaths = null;
                    }
                }
            }
        }
        this.firstLevelIncludeNames = firstLevelIncludePaths;
        
        // OAK-5169:
        // excludedParentPaths could in theory be a large list, in which case
        // the excludes() algorithm becomes non-performing. Reason is, that it
        // iterates through the changeSet and then through the excludePaths.
        // which means it becomes an O(n*m) operation.
        // This should be avoided and one way to avoid this is to make an
        // unprecise exclude filtering, where a smaller number of parent
        // exclude paths is determined - and if the change is within this
        // unprecise set, then we have to include it (with the risk of
        // false negative) - but if the change is outside of this unprecise
        // set, then we are certain that we are not excluding it.
        // one way this unprecise filter can be implemented is by 
        // starting off with eg 6 levels deep paths, and check if that brings
        // down the number far enough (to eg 11), if it's still too high,
        // cut off exclude paths at level 5 and repeat until the figure ends
        // up under 11.
        this.excludePathPatterns = new HashSet<Pattern>();
        this.unpreciseExcludePathPatterns = new HashSet<Pattern>();
        if (excludedParentPaths.size() < maxExcludedPaths) {
            for (String aRawExcludePath : excludedParentPaths) {
                this.excludePathPatterns.add(asPattern(concat(aRawExcludePath, "**")));
            }
        } else {
            final Set<String> unprecisePaths = unprecisePaths(excludedParentPaths, maxExcludedPaths, MAX_EXCLUDE_PATH_CUTOFF_LEVEL);
            for (String anUnprecisePath : unprecisePaths) {
                this.unpreciseExcludePathPatterns.add(asPattern(concat(anUnprecisePath, "**")));
            }
        }
        this.propertyNames = propertyNames == null ? null : new HashSet<String>(propertyNames);
        this.parentNodeTypes = parentNodeTypes == null ? null : new HashSet<String>(parentNodeTypes);
        this.parentNodeNames = parentNodeNames == null ? null : new HashSet<String>(parentNodeNames);
    }
    
    private String firstLevelName(String path) {
		return null;
	}

    private Set<String> unprecisePaths(Set<String> paths, int maxExcludedPaths, int maxExcludePathCutOffLevel) {
		return null;
	}

    private Set<String> unprecisePaths(Set<String> paths, int level) {
		return null;
	}

    /** for testing only **/
    public Set<String> getRootIncludePaths() {
		return null;
	}

    private Pattern asPattern(String patternWithGlobs) {
		return null;
	}

    @Override
    public boolean excludes(ChangeSet changeSet) {
		return false;
	}
    
    private boolean doExcludes(ChangeSet changeSet) {
		return false;
	}

    private static boolean patternsMatch(Set<Pattern> pathPatterns, String path) {
		return false;
	}
}
