/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.commons.providers.base;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.locks.Lock;

import org.apache.sling.commons.scheduler.Scheduler;
import org.apache.sling.discovery.DiscoveryService;
import org.apache.sling.discovery.InstanceDescription;
import org.apache.sling.discovery.TopologyEvent;
import org.apache.sling.discovery.TopologyEvent.Type;
import org.apache.sling.discovery.TopologyEventListener;
import org.apache.sling.discovery.commons.providers.BaseTopologyView;
import org.apache.sling.discovery.commons.providers.EventHelper;
import org.apache.sling.discovery.commons.providers.ViewStateManager;
import org.apache.sling.discovery.commons.providers.spi.ClusterSyncService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The ViewStateManager is at the core of managing TopologyEventListeners,
 * the 'view state' (changing vs changed) and sending out the appropriate
 * and according TopologyEvents to the registered listeners.
 * <p>
 * Note re synchronization: this class rquires a lock object to be passed
 * in the constructor - this will be applied to all public methods
 * appropriately. Additionally, the ClusterSyncService callback will
 * also be locked using the provided lock object.
 */
public class ViewStateManagerImpl implements ViewStateManager {

    private static final Logger logger = LoggerFactory.getLogger(ViewStateManagerImpl.class);
    
    /** 
     * List of bound listeners that have already received their INIT event - others are in unInitializedEventListeners.
     * @see ViewStateManagerImpl#unInitializedEventListeners
     */
    private List<TopologyEventListener> eventListeners = new ArrayList<TopologyEventListener>();

    /**
     * List of bound listeners that have not yet received their TOPOLOGY_INIT event - 
     * once they are sent the TOPOLOGY_INIT event they are moved to eventListeners (and stay there).
     * <p>
     * This list becomes necessary for cases where the bind() happens before activate, or after activate but at a time
     * when the topology is TOPOLOGY_CHANGING - at which point an TOPOLOGY_INIT event can not yet be sent.
     * @see ViewStateManagerImpl#eventListeners
     */
    private List<TopologyEventListener> unInitializedEventListeners = new ArrayList<TopologyEventListener>();
    
    /** 
     * Set true when the bundle.activate() was called, false if not yet or when deactivate() is called.
     * <p>
     * This controls whether handleChanging() and handleNewView() should cause any events
     * to be sent - which they do not if called before handleActivated() (or after handleDeactivated())
     * @see ViewStateManagerImpl#handleActivated()
     * @see ViewStateManagerImpl#handleChanging()
     * @see ViewStateManagerImpl#handleNewView(BaseTopologyView)
     * @see ViewStateManagerImpl#handleDeactivated()
     */
    private boolean activated;
    
    /**
     * Represents the 'newView' passed to handleNewTopologyView at the most recent invocation.
     * <p>
     * This is used for:
     * <ul>
     *  <li>sending with the TOPOLOGY_INIT event to newly bound listeners or at activate time</li>
     *  <li>sending as oldView (marked not current) with the TOPOLOGY_CHANGING event</li>
     *  <li>sending as oldView (marked not current in case handleChanging() was not invoked) with the TOPOLOGY_CHANGED event</li>
     * </ul>
     */
    private BaseTopologyView previousView;
    
    /**
     * Set to true when handleChanging is called - set to false in handleNewView.
     * When this goes true, a TOPOLOGY_CHANGING is sent.
     * When this goes false, a TOPOLOGY_CHANGED is sent.
     */
    private boolean isChanging;

    /**
     * The lock object with which all public methods are guarded - to be provided in the constructor.
     */
    protected final Lock lock;

    /**
     * An optional ClusterSyncService can be provided in the constructor which, when set, will
     * be invoked upon a new view becoming available (in handleNewView) and the actual
     * TOPOLOGY_CHANGED event will only be sent once the ClusterSyncService.sync method
     * does the according callback (which can be synchronous or asynchronous again).
     */
    private final ClusterSyncService consistencyService;
    
    /** 
     * A modification counter that increments on each of the following methods:
     * <ul>
     *  <li>handleActivated()</li>
     *  <li>handleDeactivated()</li>
     *  <li>handleChanging()</li>
     *  <li>handleNewView()</li>
     * </ul>
     * with the intent that - when a consistencyService is set - the callback from the
     * ClusterSyncService can check if any of the above methods was invoked - and if so,
     * it does not send the TOPOLOGY_CHANGED event due to those new facts that happened
     * while it was synching with the repository.
     */
    private int modCnt = 0;

    /** SLING-4755 : reference to the background AsyncEventSender. Started/stopped in activate/deactivate **/
    private AsyncEventSender asyncEventSender;
    
    /** SLING-5030 : this map contains the event last sent to each listener to prevent duplicate CHANGING events when scheduler is broken**/
    private Map<TopologyEventListener,TopologyEvent.Type> lastEventMap = new HashMap<TopologyEventListener, TopologyEvent.Type>();

    private MinEventDelayHandler minEventDelayHandler;

    /**
     * Creates a new ViewStateManager which synchronizes each method with the given
     * lock and which optionally uses the given ClusterSyncService to sync the repository
     * upon handling a new view where an instances leaves the local cluster.
     * @param lock the lock to be used - must not be null
     * @param consistencyService optional (ie can be null) - the ClusterSyncService to 
     * sync the repository upon handling a new view where an instances leaves the local cluster.
     */
    ViewStateManagerImpl(Lock lock, ClusterSyncService consistencyService) {
        if (lock==null) {
            throw new IllegalArgumentException("lock must not be null");
        }
        this.lock = lock;
        this.consistencyService = consistencyService;
    }

    @Override
    public void installMinEventDelayHandler(DiscoveryService discoveryService, Scheduler scheduler, long minEventDelaySecs) {
	}
    
    protected boolean hadPreviousView() {
		return false;
	}
    
    protected boolean unchanged(BaseTopologyView newView) {
		return false;
	}
    
    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#bind(org.apache.sling.discovery.TopologyEventListener)
     */
    @Override
    public void bind(final TopologyEventListener eventListener) {
	}
    
    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#unbind(org.apache.sling.discovery.TopologyEventListener)
     */
    @Override
    public boolean unbind(final TopologyEventListener eventListener) {
		return false;
	}
    
    private void enqueue(final TopologyEventListener da, final TopologyEvent event, boolean logInfo) {
	}

    /** Internal helper method that sends a given event to a list of listeners **/
    private void enqueueForAll(final List<TopologyEventListener> audience, final TopologyEvent event) {
	}
    
    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#handleActivated()
     */
    @Override
    public void handleActivated() {
	}

    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#handleDeactivated()
     */
    @Override
    public void handleDeactivated() {
	}
    
    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#handleChanging()
     */
    @Override
    public void handleChanging() {
	}
    
    /* (non-Javadoc)
     * @see org.apache.sling.discovery.commons.providers.impl.ViewStateManager#handleNewView(org.apache.sling.discovery.commons.providers.BaseTopologyView)
     */
    @Override
    public void handleNewView(final BaseTopologyView newView) {
	}

    boolean handleNewViewNonDelayed(final BaseTopologyView newView) {
		return false;
	}

    protected boolean onlyDiffersInProperties(BaseTopologyView newView) {
		return false;
	}
    
    private void doHandleConsistent(BaseTopologyView newView) {
	}

    /** get-hook for testing only! **/
    AsyncEventSender getAsyncEventSender() {
		return null;
	}

    @Override
    public int waitForAsyncEvents(long timeout) {
		return 0;
	}
    
    private int getInFlightAsyncEventCnt() {
		return 0;
	}
    
}
